// Expander.cpp : Defines the entry point for the application.
//

/** \file Expander.cpp
	\brief An example file that makes use of ExpandLib.
*/

#pragma warning(disable: 4786)

#include "expandlib/expandlib-expandable.h"

#include "expandlib/expandlib-vartype-bool.h"
#include "expandlib/expandlib-vartype-numeric.h"
#include "expandlib/expandlib-vartype-text.h"
#include "expandlib/expandlib-vartype-time.h"
#include "expandlib/expandlib-vartype-date.h"
#include "expandlib/expandlib-vartype-list.h"

#include <iostream>

using namespace std;
using namespace expander;

#include <windows.h>
#include <tchar.h>

/// The number of iterations the loop in main has to go through.
const int NUMITERATIONS=1000;

#ifdef _UNICODE
  /// Defines the string type used
  typedef wstring strtype;
# define cout wcout
#else
  typedef string strtype;
#endif

/** \name Vartype typedefs
    This was done so I don't have to use such a long name when using these types, as well as to provide an easy way to change the string type.
*/
//@{
typedef expvartype_text<strtype> EVText;
typedef expvartype_bool<strtype> EVBool;
typedef expvartype_numeric<strtype> EVNum;
typedef expvartype_time<strtype> EVTime;
typedef expvartype_date<strtype> EVDate;
typedef expvartype_list<strtype> EVTextList;
//@}

#include "tagz.h"

map<string,string> tagz_vars;

const T_CHAR* my_TAGFUNC(const T_CHAR * tag,void * p);

/// The "main" function.
int main()
{
	// The "expandable", class containing the "to be expanded" string (the source string), as well as a map containing commands
	expandable<strtype> Expje;

	// Destination string
	strtype d;

	// Map containing the different vars
	expandable<strtype>::FVarMapType vars;

	// Results of the expand
	expandresults results;

	// Some vars to measure the performance (not needed)
	LARGE_INTEGER start, end, freq;

	// Some example strings to play with (you need a source and destination string, these are source strings)
//	Expje.set_expstr(_T("${sdate} at ${stime} - This (#if(${muted};==;true;Damn, it's muted;Luckily this isn't muted)) #if(Hallo;==;hallo;gelijk;ongelijk) is:%20100\\% music, at \\$${srate}Hz on my ${dev} in #if(${nchannels};==!;2;#if(${msg};${msg}, this is a test, ;)Stereo, yeah, that's right;#if(${nchannels};==!;1;#if(${msg};${msg} ;)Mono;#if(${nchannels};>!;2;#ltrim(${nchannels};0) channels;zero channels)))!"));
//	Expje.set_expstr(_T("#if(${nchannels}==2;Stereo;Mono)"));
//	Expje.set_expstr(_T("${title}"));
	Expje.set_expstr(_T("#if(${title};${album}\\\\\\\\${artist} - #fpad(${tracknumber};2;0) - ${title};${path} - ${filename})"));

	// The set of variables that is used by the first two source strings above (you need to fill the vars map)
/*	vars[_T("srate")]=new EVText(_T("44100"));
	vars[_T("nchannels")]=new EVText(_T("2"));
	vars[_T("dev")]=new EVText(_T("EWS88MT"));
	vars[_T("msg")]=new EVText(_T(""));
	vars[_T("muted")]=new EVBool(_T("yes"));
	vars[_T("stime")]=new EVTime(_T("04:1"));
	vars[_T("sdate")]=new EVDate(_T("2002-07-31"));
*/
	// The set of variables that is used by the last source string above (you need to fill the vars map)
	vars[_T("filename")]=new EVText(_T("02 - Requiem For Evita.ape"));
	vars[_T("path")]=new EVText(_T("I:\\Audio\\Andrew Lloyd Webber\\Evita (Disc 1)"));
	vars[_T("title")]=new EVText(_T("Requiem For Evita"));
	vars[_T("album")]=new EVText(_T("Evita (Disc 1)"));
//	vars[_T("tracknumber")]=new EVText(_T("2"));
	vars[_T("tracknumber")]=new EVNum(2);
	vars[_T("artist")]=new EVText(_T("Andrew Lloyd Webber"));
	// Some examples of how to use the text list type (btw, I have nothing to do with this song :)
/*
	{
		vector<string> art;
		art.push_back(_T("Andrew Lloyd Webber"));
		art.push_back(_T("Jasper van de Gronde"));
		vars[_T("artist")]=new EVTextList(art.begin(), art.end());
	}
*//*
	{
		_TCHAR * artists[]={_T("Andrew Lloyd Webber"),_T("Jasper van de Gronde")};
		vars[_T("artist")]=new EVTextList(artists, artists+(sizeof(artists)/sizeof(artists[0])));
	}
*//*
	vars[_T("artist")]=new EVTextList(5,_T("Hello"));
*/

	// Just letting the user know what those ugly, escaped strings above look like in real life
	cout << _T("Original: ") << endl;
	cout << Expje.get_expstr() << endl;
	cout << endl;

	// Get the start time
	QueryPerformanceCounter(&start);

	{	// Perform some expands (at least one expand would be recommended :)
		for(int t=0; t<NUMITERATIONS; ++t) {
			d=_T("");
			results.Reset();
			Expje.expandto(d,vars,results);
		}
	}

	// Get the end time, as well as the frequency
	QueryPerformanceCounter(&end);
	QueryPerformanceFrequency(&freq);

	// Produce the output as well as some statistics (these do not count the calls made by, for example, knownvars)
	cout << _T("After parsing: ") << endl;
	cout << d << endl;
	cout << endl;
	cout << _T("Variables processed: ") << results.TotalVars() << endl;
	cout << _T("Variables found:     ") << results.FoundVars() << endl;
	cout << _T("Total duration:      ") << (double(end.QuadPart-start.QuadPart) / double(freq.QuadPart)) << endl;
	cout << _T("Duration per expand: ") << (double(end.QuadPart-start.QuadPart) / double(freq.QuadPart)) / double(NUMITERATIONS) << endl;
	cout << endl;

	// Don't forget to clean up!
	for(expandable<strtype>::FVarMapType::iterator it=vars.begin(); it!=vars.end(); ++it) {
		delete (*it).second;
	}



	// Now test TAGZ

	char * temp=NULL;

	tagz_vars[_T("filename")]=_T("02 - Requiem For Evita.ape");
	tagz_vars[_T("path")]=_T("I:\\Audio\\Andrew Lloyd Webber\\Evita (Disc 1)");
	tagz_vars[_T("title")]=_T("Requiem For Evita");
	tagz_vars[_T("album")]=_T("Evita (Disc 1)");
	tagz_vars[_T("tracknumber")]="2";
	tagz_vars[_T("artist")]=_T("Andrew Lloyd Webber");

	// Get the start time
	QueryPerformanceCounter(&start);

	{	// Perform some expands (at least one expand would be recommended :)
		for(int t=0; t<NUMITERATIONS; ++t) {
			if ( temp ) free(temp);
			temp=tagz_format_r("$if(%title%,%album%\\\\\\\\%artist% - $num(%tracknumber%,2) - %title%,%path% - %filename%)",my_TAGFUNC,NULL,NULL);
		}
	}

	// Get the end time, as well as the frequency
	QueryPerformanceCounter(&end);
	QueryPerformanceFrequency(&freq);

	// Produce the output as well as some statistics (these do not count the calls made by, for example, knownvars)
	cout << _T("After parsing: ") << endl;
	cout << temp << endl;
	cout << endl;
	cout << _T("Total duration:      ") << (double(end.QuadPart-start.QuadPart) / double(freq.QuadPart)) << endl;
	cout << _T("Duration per expand: ") << (double(end.QuadPart-start.QuadPart) / double(freq.QuadPart)) / double(NUMITERATIONS) << endl;
	cout << endl;

	free(temp);


	return 0;
}


const T_CHAR* my_TAGFUNC(const T_CHAR * tag,void * p)
{
	return tagz_vars[tag].c_str();
}
